/*
LCD SC2004 only lcd.c
LCD drive
	Pin assignment
	13:RA0:O:LCD E
	11:RA2:O:LCD RS
	10:RC0:O:LCD D4
   	 9:RC1:0:LCD D5
	 8:RC2:O:LCD D6
	 7:RC3:O:LCD D7
	LCD R/W is only VSS
	DB0 to DB3 are must be open
*/

#include <htc.h>
#include	"lcd.h"

static bit LCD_RS	@ ((unsigned)&PORTA*8+2);	// Register select (RA2)
static bit LCD_EN	@ ((unsigned)&PORTA*8+0);	// Enable (RA1)
#define	LCD_STROBE	(((LCD_EN = 1),(LCD_EN=0))	// spec. min 500ns


// write a byte to the LCD in 4 bit mode

void lcd_data_byteset(unsigned char c){
	PORTC = (PORTC & 0b11110000) |  (c & 0b00001111);		//Only low 4bit DATA
	LCD_STROBE;
}

void lcd_data_set(unsigned char c){
	PORTC = (PORTC & 0b11110000) | ( (c >>4 ) & 0b00001111 ) ;	//Hi SIDE DATA
	LCD_STROBE;
	PORTC = (PORTC & 0b11110000) |  (c & 0b00001111 );		//low SIDE DATA
	LCD_STROBE;
}

void
lcd_write(unsigned char c)
{
	LCD_RS = 0;
	PORTC = (PORTC & 0b11110000) | ( (c >>4 ) & 0b00001111 ) ;	//Hi SIDE DATA
	LCD_STROBE;
	PORTC = (PORTC & 0b11110000) |  (c & 0b00001111 );		//low SIDE DATA
	LCD_STROBE;
	__delay_us(40);	
}

/*
 * 	Clear and home the LCD
 */

void
lcd_clear(void)
{
	lcd_write(0x1);
	lcd_loccnt=0;
	__delay_ms(2); //PICC Pro
}

//write a string of chars to the LCD
void
lcd_puts(const char * s)
{
	LCD_RS = 1;	// write characters
	while(*s)
		lcd_putch(*s++);
}

//write one character to the LCD and auto increment location counter*/

void
lcd_putch(unsigned char c)
{
	LCD_RS = 1;	// write characters
	lcd_data_set(c);
	__delay_us(40);

	lcd_loccnt++;
	if		(lcd_loccnt == LCD_end_loc1+1){;	//1sڂȂ2sڂ
		lcd_goto(LCD_start_loc2);
		return;
	}
	else if	(lcd_loccnt == LCD_end_loc2+1){;	//2sڂȂ3sڂ
		lcd_goto(LCD_start_loc3);
		return;
	}
	else if	(lcd_loccnt == LCD_end_loc3+1){;	//3sڂȂ4sڂ
		lcd_goto(LCD_start_loc4);
		return;
	}
	else if	(lcd_loccnt == LCD_end_loc4+1){;	//4sڂȂ1sڂ
		lcd_goto(LCD_start_loc1);
		return;
	}	
}


//write to CG RAM 
void
lcd_cgwrite(unsigned char c)
{
	LCD_RS = 1;	// write characters
	lcd_data_set(c);
	__delay_us(40);
}

//Go to the specified position

void
lcd_goto(unsigned char pos)
{
	lcd_write(0x80+pos);
	lcd_loccnt=pos;
}
	
//initialise the LCD - put into 4 bit mode
void
lcd_init(void)
{
	LCD_RS = 0;				// write control bytes
	__delay_ms(15);			// power on delay (PICC Pro)
	lcd_data_byteset(0x03);	// attention!
	__delay_ms(5);			//wait
	LCD_STROBE;				//strobe again
	__delay_us(100);		//wait
	LCD_STROBE;				//strobe again
	__delay_ms(5);			//wait
	lcd_data_byteset(0x02) ;// set 4 bit mode
	__delay_us(40);			//wait
	lcd_write(0x28);		// 4 bit mode, 1/16 duty, 5x8 font
	lcd_write(0x08);		// display off
	lcd_write(0x0F);		// display on, blink curson on
	lcd_write(0x06);		// entry mode
	lcd_loccnt=0;
}

//Newline processing
void lcd_newline(unsigned char mode){
	switch (mode){
		case 1://sXy[X
		if		(lcd_loccnt < LCD_end_loc1+1){
			lcd_space_full(LCD_end_loc1-lcd_loccnt+1);//1s
			return;
		}
		else if	(lcd_loccnt < LCD_end_loc3+1){
			lcd_space_full(LCD_end_loc3-lcd_loccnt+1);//3s
			return;
		}
		else if	(lcd_loccnt < LCD_end_loc2+1){
			lcd_space_full(LCD_end_loc2-lcd_loccnt+1);//2s
			return;
		}
		else if	(lcd_loccnt < LCD_end_loc4+1){
			lcd_space_full(LCD_end_loc4-lcd_loccnt+1);//4s
			return;
		}else return;

		default://Ps
		if		(lcd_loccnt < LCD_end_loc1+1){
			lcd_goto(LCD_start_loc2);	//1sڂȂ2sڂ
			return;
		}
		else if	(lcd_loccnt < LCD_end_loc3+1){
			lcd_goto(LCD_start_loc4);	//3sڂȂ4sڂ
			return;
		}
		else if	(lcd_loccnt < LCD_end_loc2+1){
			lcd_goto(LCD_start_loc3);	//2sڂȂ3sڂ
			return;
		}
		else if	(lcd_loccnt < LCD_end_loc4+1){
			lcd_goto(LCD_start_loc1);	//4sڂȂ1sڂ
			return;
		}
	}
}	


void lcd_space_full(unsigned char count){//spacing
	while(count--){
		lcd_putch(' ');//space character write
	};
}		
		
//J[\
void lcd_move(unsigned char arrow){
	switch (arrow){
		case UP:	
		if		(lcd_loccnt < LCD_end_loc1+1)	lcd_goto(lcd_loccnt+0x54);
		else if	(lcd_loccnt < LCD_end_loc3+1)	lcd_goto(lcd_loccnt+0x2c);
		else if	(lcd_loccnt < LCD_end_loc2+1)	lcd_goto(lcd_loccnt-0x40);
		else if	(lcd_loccnt < LCD_end_loc4+1)	lcd_goto(lcd_loccnt-0x40);
		break;

		case DOWN:	
		if		(lcd_loccnt < LCD_end_loc1+1)	lcd_goto(lcd_loccnt+0x40);
		else if	(lcd_loccnt < LCD_end_loc3+1)	lcd_goto(lcd_loccnt+0x40);
		else if	(lcd_loccnt < LCD_end_loc2+1)	lcd_goto(lcd_loccnt-0x2c);
		else if	(lcd_loccnt < LCD_end_loc4+1)	lcd_goto(lcd_loccnt-0x54);
		break;
		
		case LEFT:
		if		(lcd_loccnt == LCD_start_loc1)	lcd_goto(LCD_end_loc4);	//1sڂȂ4sڂ
		else if	(lcd_loccnt == LCD_start_loc2)	lcd_goto(LCD_end_loc1);	//2sڂȂ1sڂ
		else if	(lcd_loccnt == LCD_start_loc3)	lcd_goto(LCD_end_loc2);	//3sڂȂ2sڂ
		else if	(lcd_loccnt == LCD_start_loc4)	lcd_goto(LCD_end_loc3);	//4sڂȂ3sڂ
		else 	lcd_goto(--lcd_loccnt);
		break;

		case RIGHT:
		if		(lcd_loccnt == LCD_end_loc1)	lcd_goto(LCD_start_loc2);	//1sڂȂ2sڂ
		else if	(lcd_loccnt == LCD_end_loc2)	lcd_goto(LCD_start_loc3);	//2sڂȂ3sڂ
		else if	(lcd_loccnt == LCD_end_loc3)	lcd_goto(LCD_start_loc4);	//3sڂȂ4sڂ
		else if	(lcd_loccnt == LCD_end_loc4)	lcd_goto(LCD_start_loc1);	//4sڂȂ1sڂ
		else 	lcd_goto(++lcd_loccnt);
		break;

		default :
		break;
	}
}